@@ -14,8 +14,6 @@
 
 package org.codehaus.gant ;
 
-import java.lang.reflect.Field ;
-
 import java.util.Iterator ;
 import java.util.List ;
 import java.util.Map ;
@@ -29,17 +27,8 @@ import org.apache.tools.ant.Project ;
 
 /**
  *  This class is a sub-class of <code>AntBuilder</code> to provide extra capabilities.  In particular, a
- *  dry-run capability, and the implementation of various levels of verbosity.
- *
- *  <p>If execution is from a command line Gant or call from a Groovy script then the class loader for all
- *  objects is a single instance of <code>org.codehaus.groovy.tools.RootLoader</code>, which already has Ant
- *  and Groovy jars in the classpath.  If however execution is from an Ant execution via the Gant Ant Task,
- *  then the classloader for the instance is an instance of <code>org.apache.tools.ant.AntClassLoader</code>
- *  with Ant and Groovy jars on the classpath BUT the class loader for the
- *  <code>org.apache.tools.ant.Project</code> instance is a simple <code>java.net.URLClassLoader</code> and
- *  does not have the necessary jars on the classpath.  When using Ant, the Ant jar has been loaded before
- *  the Groovy aspects of the classpath have been set up.  So we must allow for a specialized constructor
- *  taking a preprepared <code>org.apache.tools.ant.Project</code> to handle this situation. </p>
+ *  dry-run capability, and things to help support interaction between Gant and the underlying
+ *  <code>Project</code>.
  *
  *  @author Russel Winder <russel.winder@concertant.com>
  */
@@ -51,6 +40,16 @@ public class GantBuilder extends AntBuilder {
   /**
    *  Constructor that specifies which <code>Project</code> to be associated with.
    *
+   *  <p>If execution is from a command line Gant or call from a Groovy script then the class loader for all
+   *  objects is a single instance of <code>org.codehaus.groovy.tools.RootLoader</code>, which already has
+   *  Ant and Groovy jars in the classpath.  If, however, execution is from an Ant execution via the Gant
+   *  Ant Task, then the classloader for the instance is an instance of
+   *  <code>org.apache.tools.ant.AntClassLoader</code> with Ant and Groovy jars on the classpath BUT the
+   *  class loader for the <code>Project</code> instance is a simple <code>java.net.URLClassLoader</code>
+   *  and does not have the necessary jars on the classpath.  When using Ant, the Ant jar has been loaded
+   *  before the Groovy aspects of the classpath have been set up.  So we must allow for a specialized
+   *  constructor (this one) taking a preprepared <code>Project</code> to handle this situation.</p>
+   * 
    *  @param project The <code>Project</code> to be associated with.
    */
   public GantBuilder ( final Project project ) { super ( project ) ; }
@@ -65,9 +64,10 @@ public class GantBuilder extends AntBuilder {
   @Override public Object invokeMethod ( final String name , final Object arguments ) {
     if ( GantState.dryRun ) {
       if ( GantState.verbosity > GantState.SILENT ) {
+        final StringBuilder sb = new StringBuilder ( ) ;
         int padding = 9 - name.length ( ) ;
         if ( padding < 0 ) { padding = 0 ; }
-        System.out.print ( "         ".substring ( 0 , padding ) + '[' + name + "] ") ;
+        sb.append ( "         ".substring ( 0 , padding ) + '[' + name + "] ") ;
         final Object[] args = (Object[]) arguments ;
         if ( args[0] instanceof Map ) {
           // NB IntelliJ IDEA complains that (Map) is not a proper cast but using the cast (Map<?,?>) here
@@ -75,13 +75,14 @@ public class GantBuilder extends AntBuilder {
           final Iterator<Map.Entry<?,?>> i = ( (Map) args[0] ).entrySet ( ).iterator ( ) ; // Unchecked conversion here.
           while ( i.hasNext ( ) ) {
             final Map.Entry<?,?> e = i.next ( ) ;
-            System.out.print ( e.getKey ( ) + " : '" + e.getValue ( ) + '\'' ) ;
-            if ( i.hasNext ( ) ) { System.out.print ( " , " ) ; }
+            sb.append ( e.getKey ( ) + " : '" + e.getValue ( ) + '\'' ) ;
+            if ( i.hasNext ( ) ) { sb.append ( " , " ) ; }
           }
-          System.out.println ( ) ;
+          sb.append ( '\n' ) ;
+          getProject ( ).log ( sb.toString ( ) ) ;
           if ( args.length == 2 ) { ( (Closure) args[1] ).call ( ) ; }
         }
-        else if ( args[0] instanceof Closure ) { System.out.println ( ) ; ( (Closure) args[0] ).call ( ) ; }
+        else if ( args[0] instanceof Closure ) { ( (Closure) args[0] ).call ( ) ; }
         else { throw new RuntimeException ( "Unexpected type of parameter to method " + name ) ; }
       }
       return null ;
@@ -89,27 +90,14 @@ public class GantBuilder extends AntBuilder {
     return super.invokeMethod ( name , arguments ) ;
   }
   /**
-   *  Method to be called to trigger setting of the message output level on the <code>AntBuilder</code>
-   *  project.  The verbosity level is determined from <code>GantState</code>.
+   *  Accessor for the logger associated with the <code>Project</code>.
+   *
+   *  @return The <code>BuildLogger</code>.
    */
   @SuppressWarnings ( "unchecked" )
-  public void setMessageOutputLevel ( ) {
-    try {
-      //  The project is a private field in AntBuilder so we have to use reflection to get at it.  Maybe it
-      //  would be easier if this were a Groovy class :-)
-      final Field field = getClass ( ).getSuperclass ( ).getDeclaredField ( "project" ) ;
-      field.setAccessible ( true ) ;
-      final Project project = (Project) field.get ( this ) ;
-      final List<? extends BuildListener> listeners = project.getBuildListeners ( ) ; // Unchecked conversion here.
-      assert listeners.size ( ) == 1 ;
-      final BuildLogger logger = (BuildLogger) listeners.get ( 0 ) ;
-      logger.setMessageOutputLevel ( GantState.verbosity ) ;
-    }
-    catch ( final NoSuchFieldException nsfe ) {
-      throw new RuntimeException ( "No field named project in GantBuilder." , nsfe ) ;
-    }
-    catch ( final IllegalAccessException iae ) {
-      throw new RuntimeException ( "Unable to access field project in GantBuilder." , iae ) ;
-    }
+  public BuildLogger getLogger ( ) {
+    final List<? extends BuildListener> listeners = getProject ( ).getBuildListeners ( ) ; // Unchecked conversion here.
+    assert listeners.size ( ) > 0 ;
+    return (BuildLogger) listeners.get ( 0 ) ;
   }
 }
