@@ -116,7 +116,7 @@ public class GantMetaClass extends DelegatingMetaClass {
     Object returnObject = null ;
     if ( methodName.equals ( "depends" ) ) {
       for ( final Object argument : arguments ) {
-        if ( argument instanceof List ) {
+        if ( argument instanceof List<?> ) {
           for ( final Object item : (List<?>) argument ) { returnObject = processArgument ( item ) ; }
         }
         else { returnObject = processArgument ( argument ) ; }
@@ -132,6 +132,16 @@ public class GantMetaClass extends DelegatingMetaClass {
         catch ( final MissingPropertyException mpe ) { /* Purposefully empty */ }
       }
       catch ( final MissingMethodException mme ) {
+        /*
+         *  This was originally part of Yoel Spotts proposal of the GANT-10 fix, but it seems superfluous.
+         *
+         *  TODO:  Remove this after the release.
+         *
+        if ( binding.getVariable ( methodName ) != null ) {
+          //  Let Script handle this if the name is in the binding.
+          throw mme ;
+        }
+        */
         try { returnObject = ( (GantBuilder) ( binding.getVariable ( "ant" ) ) ).invokeMethod ( methodName , arguments ) ; }
         catch ( final BuildException be ) {
           //  This BuildException could be a real exception due to a failed execution of a found Ant task
