@@ -37,6 +37,7 @@ import org.apache.tools.ant.ProjectHelper ;
  *  @author Russel Winder
  */
 public class Gant_Test extends TestCase {
+  private final String endOfTargetMarker = "------ " ;
   private final String separator = System.getProperty ( "file.separator" ) ;
   private final boolean isWindows = System.getProperty ( "os.name" ).startsWith ( "Windows" ) ;
   private final String path ; {
@@ -69,7 +70,7 @@ public class Gant_Test extends TestCase {
     super.setUp ( ) ;
     project = new Project ( ) ;
     project.init ( ) ;
-    ProjectHelper.getProjectHelper ( ).parse ( project , antFile ) ;
+    ProjectHelper.configureProject ( project , antFile ) ;
     returnValue = "" ;
   }
 
@@ -138,7 +139,7 @@ public class Gant_Test extends TestCase {
       try {
         final BufferedReader br = new BufferedReader ( new InputStreamReader ( is ) ) ;
         while ( true ) {
-          final String line = br.readLine ( ) ;  //  Could throw an IOException hence the try block.
+          final String line = br.readLine ( ) ;  //  Can throw an IOException hence the try block.
           if ( line == null ) { break ; }
           sb.append ( line ).append ( '\n' ) ;
         }
@@ -147,7 +148,8 @@ public class Gant_Test extends TestCase {
     }
   }
   /*
-   *  Run Ant in a separate process.  Return the string that results.
+   *  Run Ant in a separate process.  Return the standard output and the standard error that results as a
+   *  List<String> with two items, item 0 is standard output and item 1 is standard error.
    *
    *  <p>This method assumes that either the environment variable ANT_HOME is set to a complete Ant
    *  installation or that the command ant (ant.bat on Windows) is in the path.</p>
@@ -158,9 +160,9 @@ public class Gant_Test extends TestCase {
    *  @param xmlFile the path to the XML file that Ant is to use.
    *  @param target the target to run, pass "" or null for the default target.
    *  @param expectedReturnCode the return code that the Ant execution should return.
-   *  @param withClasspath whether the Ant execution should use the full classpathso as to find all the classes.
+   *  @param withClasspath whether the Ant execution should use the full classpath so as to find all the classes.
    */
-  private String runAnt ( final String xmlFile , final String target , final int expectedReturnCode , final boolean withClasspath ) {
+  private List<String> runAnt ( final String xmlFile , final String target , final int expectedReturnCode , final boolean withClasspath ) {
     final List<String> command = new ArrayList<String> ( ) ;
     final String antHomeString = System.getenv ( "ANT_HOME" ) ;
     String antCommand ;
@@ -196,13 +198,19 @@ public class Gant_Test extends TestCase {
       catch ( final InterruptedException ignore ) { fail ( "Got an InterruptedException waiting for the output gobbler to terminate." ) ; }
       try { errorGobbler.join ( ) ;}
       catch ( final InterruptedException ignore ) { fail ( "Got an InterruptedException waiting for the error gobbler to terminate." ) ; }
-      System.err.println ( errorStringBuilder ) ;
-      return outputStringBuilder.toString ( ) ;
+      final List<String> returnList = new ArrayList<String> ( ) ;
+      returnList.add ( outputStringBuilder.toString ( ) ) ;
+      returnList.add ( errorStringBuilder.toString ( ) ) ;
+      return returnList ;
     }
     catch ( final IOException ignore ) { fail ( "Got an IOException from starting the process." ) ; }
     //  Keep the compiler happy, it doesn't realize that execution cannot get here -- i.e. that fail is a non-returning function.
     return null ;
   }
+  /**
+   *  The output due to the targets in commonBits.xml.
+   */
+  private final String commonTargetsList = "-initializeWithGroovyHome:\n\n-initializeNoGroovyHome:\n\n-initialize:\n\n-defineGantTask:\n\n" ;
   /*
    *  Tests stemming from GANT-19 and relating to ensuring the right classpath when loading the Groovyc Ant
    *  task.
@@ -211,7 +219,9 @@ public class Gant_Test extends TestCase {
     final StringBuilder sb = new StringBuilder ( ) ;
     sb.append ( "Buildfile: " ) ;
     sb.append ( path ).append ( separator ) ;
-    sb.append ( "gantTest.xml\n\n-initializeWithGroovyHome:\n\n-initializeNoGroovyHome:\n\ngantTestDefaultFileDefaultTarget:\n" ) ;
+    sb.append ( "gantTest.xml\n\n" ) ;
+    sb.append ( commonTargetsList ) ;
+    sb.append ( "gantTestDefaultFileDefaultTarget:\n" ) ;
     return sb.toString ( ) ;
   }
   private String trimTimeFromSuccessfulBuild ( final String message ) {
@@ -219,26 +229,36 @@ public class Gant_Test extends TestCase {
   }
   public void testRunningAntFromShellFailsNoClasspath ( ) {
     //  On Windows the ant.bat file always returns zero :-(
-    assertEquals ( createBaseMessage ( ) , runAnt ( antFile.getPath ( ) , null , ( isWindows ? 0 : 1 ) , false ) ) ;
+    final List<String> result = runAnt ( antFile.getPath ( ) , null , ( isWindows ? 0 : 1 ) , false ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( createBaseMessage ( ) , result.get ( 0 ) ) ;
+    final String errorResult = result.get ( 1 ) ;
+    assertTrue ( errorResult.startsWith ( "\nBUILD FAILED\n" ) ) ;
+    assertTrue ( errorResult.contains ( "org.codehaus.groovy.control.MultipleCompilationErrorsException: startup failed" ) ) ;
+    assertTrue ( errorResult.contains ( "build: 15: unable to resolve class org.codehaus.gant.ant.tests.Gant_Test\n @ line 15, column 1.\n" ) ) ;
   }
   public void testRunningAntFromShellSuccessful ( ) {
-    assertEquals ( createBaseMessage ( ) + "\nBUILD SUCCESSFUL\n\n", trimTimeFromSuccessfulBuild ( runAnt ( antFile.getPath ( ) , null , 0 , true ) ) ) ;
+    final List<String> result = runAnt ( antFile.getPath ( ) , null , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( createBaseMessage ( ) + "test:\n" + endOfTargetMarker + "test\n\nBUILD SUCCESSFUL\n\n", trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "" , result.get ( 1 ) ) ;
   }
   /*
    *  The following tests are based on the code presented in email exchanges on the Groovy developer list by
    *  Chris Miles.  cf.  GANT-50.  This assumes that the tests are run from a directory other than this one.
    */
-  private String basedirAntFilePath = path + separator + "basedir.xml" ;
-  //
-  //  TODO: The includeTag is needed because of an error -- it should be removed and the [groovy] tag always present.
-  //
-  private String createMessageStart ( final String target ) {
+  private final String basedirAntFilePath = path + separator + "basedir.xml" ;
+
+  private String createMessageStart ( final String target , final String taskName , final boolean extraClassPathDefinition ) {
     final StringBuilder sb = new StringBuilder ( ) ;
     sb.append ( "Buildfile: " ) ;
     sb.append ( path ).append ( separator ) ;
     sb.append ( "basedir.xml\n     [echo] basedir::ant basedir=" ) ;
     sb.append ( absolutePath ) ;
-    sb.append ( "\n\n-initializeWithGroovyHome:\n\n-initializeNoGroovyHome:\n\n-defineTasks:\n\n" ) ;
+    sb.append ( "\n\n-initializeWithGroovyHome:\n\n-initializeNoGroovyHome:\n\n-initialize:\n\n-define" ) ;
+    sb.append ( taskName ) ;
+    sb.append ( "Task:\n\n" ) ;
+    if ( extraClassPathDefinition ) { sb.append ( "-defineClasspath:\n\n" ) ; }
     sb.append ( target ) ;
     sb.append ( ":\n" ) ;
     return sb.toString ( ) ;
@@ -246,46 +266,110 @@ public class Gant_Test extends TestCase {
   public void testBasedirInSubdirDefaultProjectForGant ( ) {
     final String target = "defaultProject" ;
     final StringBuilder sb = new StringBuilder ( ) ;
-    sb.append ( createMessageStart ( target ) ) ;
+    sb.append ( createMessageStart ( target , "Groovy" , true ) ) ;
     sb.append ( "   [groovy] basedir::groovy basedir=" ) ;
     sb.append ( absolutePath ) ;
-    sb.append ( "\n   [groovy] basedir::gant basedir=" ) ;
+    sb.append ( "\n   [groovy] default:\n   [groovy] \n   [groovy] basedir::gant basedir=" ) ;
     //
     //  Currently a Gant object instantiated in a Groovy task in an Ant script does not inherit the basedir
     //  of the "calling" Ant.  Instead it assumes it is rooted in the process start directory.  According to
     //  GANT-50 this is an error.  The question is to decide whether it is or not.
     //
-    //  TODO : Should this be sb.append ( absolutePath ) ?  cf. GANT-50. 
+    //  TODO : Should this be sb.append ( absolutePath ) ?  cf. GANT-50.
     //
     sb.append ( System.getProperty ( "user.dir" ) ) ;
-    sb.append ( "\n\nBUILD SUCCESSFUL\n\n" ) ;
-    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( runAnt ( basedirAntFilePath , target , 0 , false ) ) ) ;
+    sb.append ( "\n   [groovy] " + endOfTargetMarker + "default\n   [groovy] \n\nBUILD SUCCESSFUL\n\n" ) ;
+    final List<String> result = runAnt ( basedirAntFilePath , target , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "" , result.get ( 1 ) ) ;
   }
   public void testBasedirInSubdirExplicitProjectForGant ( ) {
     final String target = "explicitProject" ;
     final StringBuilder sb = new StringBuilder ( ) ;
-    sb.append ( createMessageStart ( target ) ) ;
+    sb.append ( createMessageStart ( target , "Groovy" , true ) ) ;
     sb.append ( "   [groovy] basedir::groovy basedir=" ) ;
     sb.append ( absolutePath ) ;
     //
     //  In this case the instantiated Gant object is connected directly to the Project object instantiated
     //  by Ant and so uses the same basedir.  However it seems that the output (and error) stream are not
-    //  routed through the bit of Ant that prefixes the output with the current task name.
+    //  routed through the bit of Ant that prefixes the output with the current task name. :-(
     //
-    //  TODO : Sort out whether Is it correct that [groovy] is not printed out at the start of this?  cf. GANT-50.
-    //
-    sb.append ( "\nbasedir::gant basedir=" ) ;
+    sb.append ( "\ndefault:\nbasedir::gant basedir=" ) ;
     sb.append ( absolutePath ) ;
-    sb.append ( "\n\nBUILD SUCCESSFUL\n\n" ) ;
-    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( runAnt ( basedirAntFilePath , target , 0 , false ) ) ) ;
+    sb.append ( "\n" + endOfTargetMarker + "default\n\nBUILD SUCCESSFUL\n\n" ) ;
+    final List<String> result = runAnt ( basedirAntFilePath , target , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "" , result.get ( 1 ) ) ;
   }
   public void testBasedirInSubdirGantTask ( ) {
     final String target = "gantTask" ;
     final StringBuilder sb = new StringBuilder ( ) ;
-    sb.append ( createMessageStart ( target ) ) ;
-    sb.append ( "     [gant] basedir::gant basedir=" ) ;
+    sb.append ( createMessageStart ( target , "Gant" , false ) ) ;
+    sb.append ( "default:\n     [gant] basedir::gant basedir=" ) ;
     sb.append ( absolutePath ) ;
-    sb.append ( "\n\nBUILD SUCCESSFUL\n\n" ) ;
-    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( runAnt ( basedirAntFilePath , target , 0 , false ) ) ) ;
+    sb.append ( "\n" + endOfTargetMarker + "default\n\nBUILD SUCCESSFUL\n\n" ) ;
+    final List<String> result = runAnt ( basedirAntFilePath , target , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "" , result.get ( 1 ) ) ;
   }
+  //
+  //  Test the GANT-80 issues.
+  //
+  public void test_GANT_80 ( ) {
+    final String antFilePath = path + separator + "GANT_80.xml" ;
+    final StringBuilder sb = new StringBuilder ( ) ;
+    sb.append ( "Buildfile: " ) ;
+    sb.append ( antFilePath ) ;
+    sb.append ( "\n\n" ) ;
+    sb.append ( commonTargetsList ) ;
+    sb.append ( "default:\ndefault:\n     [gant] From println.\n     [gant] On standard error.\n     [echo] From ant.echo.\n" + endOfTargetMarker + "default\n\nBUILD SUCCESSFUL\n\n" ) ;
+    final List<String> result = runAnt ( antFilePath , null , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "" , result.get ( 1 ) ) ;
+  }
+  //
+  //  Ensure that errors are handled correctly by checking one error return case.
+  //
+  public void testGantTaskErrorReturn ( ) {
+    final File file = new File ( path , "testErrorCodeReturns.xml" ) ;
+    final String target = "usingGantAntTask" ;
+    final StringBuilder sb = new StringBuilder ( ) ;
+    sb.append ( "Buildfile: " ) ;
+    sb.append ( file.getPath ( ) ) ;
+    sb.append ( "\n\n" ) ;
+    sb.append ( commonTargetsList ) ;
+    sb.append ( target ) ;
+    sb.append ( ":\n" ) ;
+    final List<String> result = runAnt ( file.getPath ( ) , target , 1 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    final String errorResult = result.get(1) ;
+    assertTrue ( errorResult.startsWith ( "\nBUILD FAILED\n" ) ) ;
+    assertTrue ( errorResult.contains ( file.getPath ( ) ) ) ;
+    assertTrue ( errorResult.contains ( "Gantfile does not exist." ) ) ;
+  }
+  /*
+   *  For the moment comment this out because there is no guarantee of a Gant installation.
+   *
+   *  TODO:  Find out how to set up a Gant installation so this can be tested.
+   *
+  public void testExecOfGantScriptReturnErrorCode ( ) {
+    final File file = new File ( path , "testErrorCodeReturns.xml" ) ;
+    final String target = "usingExec" ;
+    final StringBuilder sb = new StringBuilder ( ) ;
+    sb.append ( "Buildfile: " ) ;
+    sb.append ( file.getPath ( ) ) ;
+    sb.append ( "\n\n" ) ;
+    sb.append ( target ) ;
+    sb.append ( ":\n     [exec] Cannot open file  nonexistentGantFile.gant\n     [echo] ErrorLevel: 253\n\nBUILD SUCCESSFUL\n\n" ) ;
+    final List<String> result = runAnt ( file.getPath ( ) , target , 0 , true ) ;
+    assert result.size ( ) == 2 ;
+    assertEquals ( sb.toString ( ) , trimTimeFromSuccessfulBuild ( result.get ( 0 ) ) ) ;
+    assertEquals ( "     [exec] Result: 253\n" , result.get ( 1 ) ) ;
+  }
+  */
 }
